/*************************************************************************************
Data Visualization - 3D Visualization Project                
Processing 3                                    
                                                     
Author: Yoko Ebata                    
Supervisor: Rodger (Jieliang) Luo, George Legrady                           

Purpose: Demonstrate a 3D visualization of nonfictional "feng shui"-related checkouts within SPL visitors across years 2008-2018  

Usage: 1. A mouse left-drag will rotate the camera around the subject.
          2. A right drag will zoom in and out. 
          3. A middle-drag (command-left-drag on mac) will pan. 
          4. A double-click restores the camera to its original position. 
          5. The shift key constrains rotation and panning to one axis or the other.
*************************************************************************************/

import peasy.*;
PeasyCam cam;
import controlP5.*;
ControlP5 cp5;

CheckBox checkbox;
Toggle toggle;
  
PFont proxima;

color clr[] = {#F89167, #F4C356, #E5DF60, #DDF774, #7AFCB6, #52F8F8, #77D4F8, #C3BDF7, #F7ABE2, #F9858D, #F9858D};

 
Table table;
int numRows;

ArrayList<String> blackListedNumbers = new ArrayList<String>(); 
ArrayList<Book> books = new ArrayList<Book>(); 
//Book is named as whatever the class name is ^ "Class_Book"

float[][] bookLoc;

void setup(){
  size(1000, 1000, P3D);
  cam = new PeasyCam(this, 350);
  gui();
  
  proxima = createFont("Proxima Nova", 64, false);
  textFont(proxima);
  
  float fov = PI/3;
  float cameraZ = (height/2.0) / tan(fov/2.0);
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*10.0); 
  
  table = loadTable("fengshui.csv");
  numRows = table.getRowCount();
  //println("Rows: " + numRows);
  //Year  Month  Title  months
  
  
  
  for(int i=1; i<numRows; i++){
    String title = table.getString(i,0);
    int year = table.getInt(i,1);
    String time = table.getString(i,2);
    int month = table.getInt(i,3);
    String dewey = table.getString(i,4);
    
    books.add(new Book(title, year, time, month, dewey));
  }
  
}

void draw(){
  background(255);
  bookLoc = new float[books.size()][2];
 
  
  //TITLE
  strokeWeight(0.5);
  fill(#846d5f);
  textSize(10);
  textAlign(CENTER);
  text("FENG SHUI PROMINENCE IN SPL", 0, -50);

  //SUBTITLE
  textSize(5);
  fill(#ceb5a5);
  text("Assorted with Various Dewey Classes", 0, -45);
  text("Data Sourced from Seattle Public Library", 0, -40);
  textSize(4);
  text("MAT 259 | YOKO EBATA", 0, -35);
  
  
  // key text
  cam.beginHUD();
 
  fill(#846d5f, 30);
  rect(width-990, height-970, 380, 190);
  
  fill(0);
  textAlign(LEFT);
  textSize(13);
  text("key", width-970, height-970);
  text("133: specific topics in parapsychology & occultism", width-970, height-950);
  text("138: physiognomy", width-970, height-935);
  text("332: financial economics", width-970, height-920);
  text("613: personal health & safety", width-970, height-905);
  text("641: food & drink", width-970, height-890);
  text("646: sewing, clothing, management of personal & family life", width-970, height-875);
  text("648: housekeeping", width-970, height-860);
  text("712: landscape architecture", width-970, height-845);
  text("747: interior decoration", width-970, height-830);
  text("789: expired music dewey class", width-970, height-815);
  text("895: literatures of east & southeast asia", width-970, height-800);
  
 
 
  //color keys
   fill(#deefed);
   rect(width-990, height-960, 10, 10);
   
   fill(#ff9400);
   rect(width-990, height-945, 10, 10);
   
   fill(#fcff79);               
   rect(width-990, height-930, 10, 10);
   
   fill(#a5d347);
   rect(width-990, height-915, 10, 10);

   fill(#478452);
   rect(width-990, height-900, 10, 10);
   
   fill(#0a5617);
   rect(width-990, height-885, 10, 10);
   
   fill(#379396);
   rect(width-990, height-870, 10, 10);
   
   fill(#003c4f);
   rect(width-990, height-855, 10, 10);
   
   fill(#001c51);
   rect(width-990, height-840, 10, 10);
   
   fill(#9b8d7f);
   rect(width-990, height-825, 10, 10);
   
   fill(#30251c);
   rect(width-990, height-810, 10, 10);
 
  
  cam.endHUD();
  
  
  for(int i=0; i<books.size(); i++){
    if(!blackListedNumbers.contains(books.get(i).dewey)){
    float yearAxis = map(Integer.parseInt((books.get(i).year)+ ""), 2008, 2018, -50, 50);
    float timeAxis = map(timeParser(books.get(i).time), 0, 86400, -100, 100);
    float monthAxis = map(books.get(i).month, 1, 12, -100, -20);
    color c = deweyDetermine(books.get(i).dewey);
    //if(i < 10 || (i < 1000 && i > 990)){
    //System.out.println("YEAR" + " " + "TIME" + " " + "MONTH");
    //System.out.println(yearAxis + " " + timeAxis + " " + monthAxis);
    //}
    
    
    
    //if(i == 1){
    //  System.out.println(yearAxis);
    //  System.out.println(timeAxis);
    //  System.out.println(monthAxis);
    //}
    
    
    
    pushMatrix();
    translate(yearAxis, timeAxis, monthAxis);
        point(0,0,0);
        float mouseObjectDistance = sq(mouseX-screenX(0, 0, 0))+sq(mouseY-screenY(0, 0, 0)); 
        if (mouseObjectDistance < 20)
          {
            stroke(c,255);
            point(0,0,0);
            stroke(0,0,0,0);
            fill(0);
            textMode(SHAPE);
            textSize(3);
            text(books.get(i).title.toUpperCase(),0,0);
          }
        
    strokeWeight(0.3);
    //stroke(c);
    fill(c, 90);
    noStroke();
    box(2);
    popMatrix();
  }

  }

  int start_date = 2008; 
  
  for(int i = 0; i < 11; i++){
    int date = start_date + i;
      
      fill(50);
      rotateX(0);
      textSize(4);
      text(date, -55 + (i * 10) , 75, -10);
  }
  
  int start_time = 8;
  
  for(int i = 0; i < 13; i++){
    int date = start_time + i;
   
      fill(50);
      rotateX(0);
      textSize(3);
      text(date+":00", -70 , 65 - (i * 7.9), -100);
  }
  
   int start_month = 1;
  
  for(int i = 0; i < 12; i++){
    int date = start_month + i;
   
     pushMatrix();
      fill(50);
      rotateX(radians(90));
      textSize(3);
      text(date, -65, -20 - (i * 7), -70);
      popMatrix();
  }
  
   rotateX(300);
   strokeWeight(1);
   stroke(0);
   translate(4, -10, 70);
   fill(#334349);
   rect(-60, 20, 110, 100);
   


  }
